from flask.ext.mongorest.resources import Resource
from flask_sparkle import documents, schemas


class VersionResource(Resource):
    document = documents.Version

class ApplicationResource(Resource):
    document = documents.Application
    schema = schemas.Application
    fields = ['name', 'slug', 'versions', 'date_updated', 'date_created']
    related_resources = {
        'versions': VersionResource
    }

    def get_object(self, pk, **kwargs):
        return self.get_queryset().get(slug=pk)



from flask_views.db.mongoengine.detail import DetailView

from flask_sparkle import sparkle
from flask_sparkle.documents import Application


class AppcastView(DetailView):
    get_fields = { 'slug': 'slug' }
    document_class = Application
    template_name = 'appcast.xml'

sparkle.add_url_rule('/<slug>/appcast.xml', view_func=AppcastView.as_view('appcast'))


try:
    from flask.ext.mongorest.views import ResourceView
    from flask_sparkle.resources import ApplicationResource
    from flask.ext.mongorest import methods
    from flask.ext.mongorest.authentication import AuthenticationBase
    from flask.ext.login import current_user


    class SparkleAuthentication(AuthenticationBase):
        def authorized(self, obj=None):
            return 'sparkle' in getattr(current_user, 'roles', [])

    class ApplicationView(ResourceView):
        resource = ApplicationResource
        methods = [methods.Create, methods.Update, methods.Fetch, methods.List]
        authentication_methods = [SparkleAuthentication]

except ImportError:
    pass



from cleancat import URL, Bool, DateTime, List, Schema, String
from cleancat.mongo import MongoEmbedded

from flask_sparkle import documents


class Version(Schema):
    date_published = DateTime(required=False)
    title = String(required=False)
    description = String(required=False)
    update_url = URL(required=False)
    version = String(required=False)
    short_version = String(required=False)
    length = String(required=False)
    dsa_signature = String(required=False)
    is_published = Bool(required=False)


class Application(Schema):
    name = String()
    slug = String()
    versions = List(MongoEmbedded(documents.Version, Version), required=False)



from flask_admin.contrib.mongoengine import ModelView

from flask_sparkle.documents import *

class ApplicationAdminView(ModelView):
    list_display = ['name', 'slug', 'latest_version_string']

def register(admin):
    admin.add_view(ApplicationAdminView(Application, name='Sparkle'))



from mongoengine import *
from flask_common.documents import DocumentBase


class Version(EmbeddedDocument):
    date_published = DateTimeField()
    title = StringField()
    description = StringField()
    update_url = URLField()
    version = StringField()
    short_version = StringField()
    length = StringField()
    dsa_signature = StringField()
    is_published = BooleanField()


class Application(DocumentBase):
    name = StringField()
    slug = StringField(unique=True)

    versions = ListField(EmbeddedDocumentField(Version))

    def latest_version(self):
        versions = self.versions
        for version in versions:
            if version.is_published:
                return version

    def latest_version_string(self):
        latest_version = self.latest_version()
        return latest_version and latest_version.version



from setuptools import setup

setup(
    name='flask-sparkle',
    version='0.3.0',
    url='http://github.com/closeio/flask-sparkle',
    license='MIT',
    author='Close.io',
    author_email='engineering@close.io',
    maintainer='Close.io',
    maintainer_email='engineering@close.io',
    description='Flask app that publishes Sparkle update feeds',
    platforms='any',
    classifiers=[
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
    ],
    packages=[
        'flask_sparkle',
    ],
    package_data={
        'flask_sparkle': ['templates/*']
    },
)
